/* Miscellaneous machine independent utilities
 */

#include <common.h>
#include "global.h"
  
/* Convert hex-ascii to integer */
int
htoi(s)
char *s;
{
    int i = 0;
    char c;
  
    while((c = *s++) != '\0'){
        if(c == 'x')
            continue;   /* allow 0x notation */
        if('0' <= c && c <= '9')
            i = (i * 16) + (c - '0');
        else if('a' <= c && c <= 'f')
            i = (i * 16) + (c - 'a' + 10);
        else if('A' <= c && c <= 'F')
            i = (i * 16) + (c - 'A' + 10);
        else
            break;
    }
    return i;
}
 
/* Convert ascii to long */
long
atol( char *s)
{
    long i = 0;
    char c;
  
    while((c = *s++) != '\0'){
        if('0' <= c && c <= '9')
            i = (i * 10) + (c - '0');
        else
            break;
    }
    return i;
}
 
int
atoi(char *s)
{
	long i;

	i = atol(s);

	return (int)i;
}

int
isalpha(int c)
{
	if((c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z'))
		return 1;
	else
		return 0;
}

int
isupper(int c)
{
	if(c >= 'A' && c <= 'Z')
		return 1;
	else
		return 0;
}

int
isdigit ( int ch ) {
    return (unsigned int)(ch - '0') < 10u;
}

int
isspace(char c)
{
	if(c == ' ' || c == '\t' || c == '\n')
		return 1;
	else
		return 0;
}

int
tolower(char c)
{
  return ((c >= 'A' && c <= 'Z') ? c + 32: c);
}

int
strcasecmp(const char *s1, const char *s2)
{
  while (*s1 != '\0' && tolower(*s1) == tolower(*s2))
    {
      s1++;
      s2++;
    }

  return tolower(*(unsigned char *) s1) - tolower(*(unsigned char *) s2);
}

int
strncasecmp(const char *s1, const char *s2, int n)
{
  if (n == 0)
    return 0;

  while (n-- != 0 && tolower(*s1) == tolower(*s2))
    {
      if (n == 0 || *s1 == '\0' || *s2 == '\0')
	break;
      s1++;
      s2++;
    }

  return tolower(*(unsigned char *) s1) - tolower(*(unsigned char *) s2);
}


/*
 * Convert a string to an unsigned long integer.
 *
 * Ignores `locale' stuff.  Assumes that the upper and lower case
 * alphabets and digits are each contiguous.
 */
unsigned long
strtoul(nptr, endptr, base)
	char *nptr;
	char **endptr;
	register int base;
{
	register const char *s = nptr;
	register unsigned long acc;
	register int c;
	register unsigned long cutoff;
	register int neg = 0, any, cutlim;

	/*
	 * See strtol for comments as to the logic used.
	 */
	do {
		c = *s++;
	} while (isspace(c));
	if (c == '-') {
		neg = 1;
		c = *s++;
	} else if (c == '+')
		c = *s++;
	if ((base == 0 || base == 16) &&
	    c == '0' && (*s == 'x' || *s == 'X')) {
		c = s[1];
		s += 2;
		base = 16;
	}
	if (base == 0)
		base = c == '0' ? 8 : 10;
	cutoff = (unsigned long)MAXINT32 / (unsigned long)base;
	cutlim = (unsigned long)MAXINT32 % (unsigned long)base;
	for (acc = 0, any = 0;; c = *s++) {
		if (isdigit(c))
			c -= '0';
		else if (isalpha(c))
			c -= isupper(c) ? 'A' - 10 : 'a' - 10;
		else
			break;
		if (c >= base)
			break;
		if (any < 0 || acc > cutoff || (acc == cutoff && c > cutlim))
			any = -1;
		else {
			any = 1;
			acc *= base;
			acc += c;
		}
	}
	if (any < 0) {
		acc = MAXINT32;
	} else if (neg)
		acc = -acc;
	if (endptr != 0)
		*endptr = (char *) (any ? s - 1 : nptr);
	return (acc);
}

char *
gets(char *s, int l)
{
	int c;
	char *cs;

	cs = s;
	while ((c = getc()) != '\r' && --l > 0) {
		switch(c) {
		case '\b':
			if (cs > s) {
				*--cs = '\0';
				putc('\b');
				putc(' ');
				putc('\b');
			}
			break;
		default:
			putc(c);
			*cs++ = c;
		}
	}
	*cs++ = '\n';
	putc('\n');
	*cs++ = '\0';
	return s;
}

/* Convert short from little-endian to big-endian
 */
unsigned short
le2bes(unsigned short les)
{
	unsigned short bes;

	bes =	((les >> 8) & 0xff) |
		((les << 8) & 0xff00);
	return bes;
}

/* Convert long from little-endian to big-endian
 */
unsigned long
le2bel(unsigned long lel)
{
	unsigned long bel;

	bel =	((lel >> 24) & 0xff) |
		((lel >> 8) & 0xff00) |
		((lel << 8) & 0xff0000) |
		((lel << 24) & 0xff000000);
	return bel;
}

